import { useState } from 'react';
import { useNavigate, Link } from "react-router-dom";
import axios from 'axios';

const Register = (): JSX.Element => {
    const navigate = useNavigate();
    const [email, setEmail] = useState('');
    const [password, setPassword] = useState('');
    const [username, setUsername] = useState('');
    const [usernameError, setUsernameError] = useState('');
    const [emailError, setEmailError] = useState('');
    const [passwordError, setPasswordError] = useState('');


    function isValidEmail(email: string) {
        const emailRegex = /^[A-Z0-9._%+-]+@[A-Z0-9.-]+\.[A-Z]{2,}$/i;
        return emailRegex.test(email);
    }

    function isValidPassword(password: string) {
        const uppercaseRegex = /[A-Z]/;
        const lowercaseRegex = /[a-z]/;
        const digitRegex = /[0-9]/;
        const specialCharRegex = /[!@#$%^&*()_+{}[\]:;<>,.?~\\-]/;
        const minLength = 8;
      
        const errors = [];

        if (password.length < minLength) {
            errors.push('Password should be at least 8 characters long.\n');
        }

        if (!uppercaseRegex.test(password)) {
            errors.push('Password should contain at least one uppercase letter.\n');
        }

        if (!lowercaseRegex.test(password)) {
            errors.push('Password should contain at least one lowercase letter.\n');
        }

        if (!digitRegex.test(password)) {
            errors.push('Password should contain at least one digit.\n');
        }

        if (!specialCharRegex.test(password)) {
            errors.push('Password should contain at least one special character.\n');
        }

        setPasswordError(errors.join(''));

        return errors.length === 0;
    }

    const userRegister = async () => {
        const validEmail = isValidEmail(email);
        const validPassword = isValidPassword(password);
        let hasErrors = false;
    
        if (!username) {
          setUsernameError('Please provide a username.');
          hasErrors = true;
        } else {
          setUsernameError('');
        }
    
        if (!email) {
          setEmailError('Please provide an email.');
          hasErrors = true;
        } else if (!validEmail) {
          setEmailError('Please provide a valid email address. Example: "User1@example.com"');
          hasErrors = true;
        } else {
          setEmailError('');
        }
    
        if (!password) {
          setPasswordError('Please provide a password.');
          hasErrors = true;
        } else if (!validPassword) {
            hasErrors = true;
        } else {
          setPasswordError('');
        }
    
        if (hasErrors) {
            return;
        }

        try {
            const response = await axios.post('http://localhost:3000/api/register', {
                email,
                username,
                password
            }, {
                headers: {
                    'Content-Type': 'application/json'
                }
            });
        
            if (response.status === 200) {
                navigate('/');
                setEmail('');
                setUsername('');
                setPassword('');
            } else {
                alert(`Registration error: ${response.data.msg}`);
            }
        } catch (error) {
            alert(`Error fetching: ${error}`);
        }
    };

    return (
    <div className="font-helvatica bg-yellowPallete h-screen flex justify-center items-center">
        <div className="bg-white p-10 rounded-lg shadow-lg">
            <h1 className="text-[24px] font-bold text-blackPallete">Register</h1>
            <form className='text-[16px]'>
            <div className="mb-4">
                <label className="text-gray block mb-2" htmlFor="username">
                    Username
                </label>
                <input
                    type="text"
                    id="username"
                    value={username}
                    onChange={(e) => setUsername(e.target.value)}
                    className="w-full p-2 border rounded"
                />
                {usernameError && (
                    <div className="text-red300 text-sm">{usernameError}</div>
                )}
            </div>
            <div className="mb-4">
                <label className="text-gray block mb-2" htmlFor="email">
                    Email
                </label>
                <input
                    type="email"
                    id="email"
                    value={email}
                    onChange={(e) => setEmail(e.target.value)}
                    className="w-full p-2 border rounded"
                />
                {emailError && (
                    <div className="text-red300 text-sm">{emailError}</div>
                )}
            </div>
            <div className="mb-2">
                <label className="text-blackPallete block mb-2" htmlFor="password">
                    Password
                </label>
                <input
                    type="password"
                    id="password"
                    value={password}
                    onChange={(e) => setPassword(e.target.value)}
                    className="w-full p-2 border rounded"
                />
                {passwordError && (
                    <div className="text-red-500 text-sm">
                        {passwordError.split('\n').map((error, index) => (
                        <div key={index}>{error}</div>
                        ))}
                    </div>
                )}
            </div>
            <div className='mb-2'>
                <p className='text-greyPallete'>Already have an account? <Link to="/" className="text-greyPallete font- hover:underline hover:text-yellowPallete">Sign In here</Link></p>
            </div>
            <button
                type="button"
                onClick={userRegister}
                className="bg-greyPallete text-white py-2 px-4 rounded hover:bg-opacity-80"
            >
                Sign Up
            </button>
            </form>
        </div>
    </div>
    )
  }
  
  export default Register
  