import NavBar from "../components/Navbar";
import { useNavigate } from "react-router-dom";
import { useEffect, useState } from 'react';
import axios from "axios";
import Cookies from 'js-cookie';
import checkToken from "../middleware/Auth";
import { toast,ToastContainer } from 'react-toastify';
import 'react-toastify/dist/ReactToastify.css';

function WriteBlog() {
  const navigate = useNavigate();

  const [content, setContent] = useState('');
  const [title, setTitle] = useState('');
  const [image, setImage] = useState<File | undefined>(undefined);
  const [isPremium, setIsPremium] = useState(false); // State untuk premium
  const [blogTitleError, setBlogTitleError] = useState('');
  const [blogContentError, setBlogContentError] = useState('');
  const [blogThumbnailError, setBlogThumbnailError] = useState('');

  useEffect(()=> {
    const isValid = checkToken();
    
    if (!isValid) {
      navigate('/');
      return;
    }
    
  }, [navigate])
  
  function handlePremiumChange(e: React.ChangeEvent<HTMLInputElement>) {
    setIsPremium(e.target.checked); // Mengubah state saat checkbox diubah
  }

  function handleContentChange(e: React.ChangeEvent<HTMLTextAreaElement>) {
    setContent(e.target.value);
  }
  
  function handleImageChange(e: React.ChangeEvent<HTMLInputElement>) {
    // Mengambil file gambar dari input
    const file = e.target.files ? e.target.files[0] : undefined;
    setImage(file);

  }

  function isValidTitleBlog(blog: FormData){
    const title = blog.get('title') as string;

    const errors = [];

    if (!title.trim()){
      errors.push('Blog should have a title.\n');
    }

    setBlogTitleError(errors.join(''));

    return errors.length === 0;

  }

  function isValidContentBlog(blog: FormData){
    const content = blog.get('content') as string;

    const errors = [];

    if (!content.trim()){
      errors.push('Blog should have a content.\n');
    }

    setBlogContentError(errors.join(''));

    return errors.length === 0;

  }

  function isValidThumbnailBlog(blog: FormData){
    const image = blog.get('file') as File | undefined;

    const errors = [];

    if (!image){
      errors.push('Blog should have a thumbnail.\n');
    }

    setBlogThumbnailError(errors.join(''));

    return errors.length === 0;

  }

  const handlePublish = async () => {
    let user_id;
    let user_name;
    try {
      const config = {
        headers: {
          Authorization: `Bearer ${Cookies.get('token')}`,
        }
      };
      const response = await axios.get('http://localhost:3000/api/user', config);
      const data = JSON.parse(JSON.stringify(response.data));
      user_id = data.user_id;
      user_name = data.user_name;
    } catch (error) {
      console.error('Error publishing blog:', error);
      toast.error('Failed to publish blog, user not found'); 
    }

    const formData = new FormData();
    formData.append('title', title);
    formData.append('content', content);
    formData.append('user_id', user_id);
    formData.append('user_name', user_name);
    formData.append("is_premium", isPremium ? "true" : "false");

    if (image) {
      formData.append('file', image);
    }
    for (const value of formData.values()) {
      console.log(value);
    }

    const validTitleBlog = isValidTitleBlog(formData);
    const validContentBlog = isValidContentBlog(formData);
    const validThumbnailBlog = isValidThumbnailBlog(formData);

    if(!validTitleBlog || !validContentBlog || !validThumbnailBlog){
      return;
    }
    

    try {
      await axios.post('http://localhost:3000/api/blogs', formData, {
        headers: {
          Authorization: `Bearer ${Cookies.get('token')}`,
          'Content-Type': 'multipart/form-data',
        },
      });

      toast.success('Blog published successfully', {
        position: "bottom-center",
        autoClose: 5000,
        hideProgressBar: false,
        closeOnClick: true,
        pauseOnHover: true,
        draggable: true,
        progress: undefined,
        theme: "light",
      });
        
      navigate('/home');
    } catch (error) {
      console.error('Error publishing blog:', error);
      toast.error('Error publishing blog', {
        position: "bottom-center",
        autoClose: 5000,
        hideProgressBar: false,
        closeOnClick: true,
        pauseOnHover: true,
        draggable: true,
        progress: undefined,
        theme:"light",
        className: "toast-message",
      });        
    }
  }

  return (
    <div className="font-serif mt-24">
      <NavBar/>
      <div className="main-cont w-[55vw] mx-auto bg-blue">
        <div className="mb-3 text-[32px] text-bold">
          <input 
            type="text" 
            className="w-[100%] text-[32px] border-0 focus:outline-none"
            placeholder="Title"
            name="title"
            onChange={(e) => setTitle(e.target.value)}
            id="title"
          />
          {blogTitleError && (
            <div className="text-red-500 text-sm mt-2 ml-4">{blogTitleError}</div>
          )}
        </div>
        <div className="mb-3 ml-3 border-solid border-1 p-3 border-greyPallete">
          <div className="text-greyPallete mb-3">
            Choose Blog Thumbnail
          </div>
          <label htmlFor="fileInput" className="block cursor-pointer">
            <input
              type="file"
              accept="image/*"
              onChange={handleImageChange}
              id="fileInput"
              className="hidden"
            />
            <div className="bg-white border-solid border-2 border-greyPallete w-full p-2 rounded-md">
              <span className="text-greyPallete block overflow-hidden overflow-ellipsis whitespace-nowrap">
                {image ? image.name : "No file chosen"}
              </span>
            </div>
          </label>
          {blogThumbnailError && (
            <div className="text-red-500 text-sm mt-2">{blogThumbnailError}</div>
          )}
        </div>

        <div className="mb-3 ml-3">
          <label>
            <input
              type="checkbox"
              checked={isPremium} 
              onChange={handlePremiumChange}
              className={`w-6 h-6 ${isPremium ? 'bg-greyPallete' : 'bg-white'} rounded-md`}
            />
            <span>
              Premium
            </span>

          </label>
        </div>
        <div className="mb-20">
          <textarea
            className="w-full h-[70vh] text-[21px] border-0 resize-none"
            placeholder="Tell your story..."
            value={content}
            onChange={handleContentChange}
          />
          {blogContentError && (
            <div className="text-red-500 text-sm mt-2 mb-3">{blogContentError}</div>
          )}
          <button 
            onClick={handlePublish} 
            className="bg-[#6b6b6b] text-[21px] bg-opacity-20 px-3 py-1"
          > Publish </button>
        </div>
      </div>
      <ToastContainer
        position="bottom-center"
        autoClose={5000}
        hideProgressBar={false}
        newestOnTop={false}
        closeOnClick
        rtl={false}
        pauseOnFocusLoss
        draggable
        pauseOnHover
        progressStyle={{ backgroundColor: 'blackPallete' }} 
      />
    </div>
  )
}

export default WriteBlog;